@@ -54,34 +54,37 @@ private:
   private:
     /// Set of LTL formulae
     std::set<class Ltl*,struct ltless> m_set;
+    /// Table of LTL formulae, indexed by Ltl::m_num
+    const class Ltl** m_table;
     /// Copy constructor
     Store (const class Store&);
     /// Assignment operator
     class Store& operator= (const class Store&);
   public:
     /// Constructor
-    Store () : m_set () {}
+    Store () : m_set (), m_table (0) {}
     /// Destructor
     ~Store () {
+      delete[] m_table;
       for (std::set<class Ltl*,struct ltless>::iterator i = m_set.begin ();
 	   i != m_set.end (); i++) {
 	delete *i;
       }
     }
     /// Insert a formula to the set
-    class Ltl& insert (class Ltl& ltl) {
-      std::pair<std::set<class Ltl*,struct ltless>::iterator,bool> p =
-	m_set.insert (&ltl);
-      if (!p.second)
-	delete &ltl;
-      return **p.first;
-    }
+    class Ltl& insert (class Ltl& ltl);
+    /// Translate a formula index to a formula reference
+    /// @param f	formula index to be translated
+    /// @return		the corresponding formula
+    const class Ltl& fetch (unsigned f) const { return *m_table[f]; }
   };
   static class Store m_store;
   friend class Store;
 public:
+  /// Number of the LTL formula
+  unsigned m_num;
   /// Constructor
-  Ltl () {}
+  Ltl () : m_num (0) {}
 private:
   /// Copy constructor
   Ltl (const class Ltl& old);
@@ -95,6 +98,10 @@ protected:
   /// @return		an equivalent formula
   static class Ltl& insert (class Ltl& ltl) { return m_store.insert (ltl); }
 public:
+  /// Translate a formula index to a formula reference
+  /// @param f		formula index to be translated
+  /// @return		the corresponding formula
+  static const class Ltl& fetch (unsigned f) { return m_store.fetch (f); }
 
   /// Returns a clone of the negation of this Ltl formula.
   virtual class Ltl& negClone () const = 0;
